@@ -24,6 +24,7 @@
  
 #include <stdlib.h>
 #include <string.h>
+#include <locale.h>
 
 #include <gtk/gtk.h>
 
@@ -238,19 +239,30 @@ void apply_screen_info_token(char *token, char *value, void *data)
  *   "mode_name"  dot_clock  timings  flags
  *
  **/
-static nvModeLinePtr modeline_parse(const char *modeline_str)
+static nvModeLinePtr modeline_parse(const char *modeline_str,
+                                    const int broken_doublescan_modelines)
 {
     nvModeLinePtr modeline = NULL;
     const char *str = modeline_str;
-    char *tmp;
-    char *tokens;
-
+    char *tmp, *old_locale;
+    char *tokens, *nptr;
+    double pclk, htotal, vtotal, factor;
 
     if (!str) return NULL;
 
     modeline = (nvModeLinePtr)calloc(1, sizeof(nvModeLine));
     if (!modeline) return NULL;
 
+    /* Make sure strtod uses . instead of , as the decimal separator */
+    old_locale = setlocale(LC_NUMERIC, NULL);
+
+    if (old_locale)
+        old_locale = strdup(old_locale);
+
+    if (!old_locale || !setlocale(LC_NUMERIC, "C"))
+        nv_warning_msg("Error parsing server modeline '%s': could not "
+                       "set the locale 'C'.", modeline_str);
+
     /* Parse the modeline tokens */
     tmp = strstr(str, "::");
     if (tmp) {
@@ -270,26 +282,10 @@ static nvModeLinePtr modeline_parse(const char *modeline_str)
     if (!str) goto fail;
 
     /* Read dot clock */
-    {
-        int digits = 100;
-
-        str = parse_read_integer(str, &(modeline->data.clock));
-        modeline->data.clock *= 1000;
-        if (*str == '.') {
-            str++;
-            while (digits &&
-                   *str &&
-                   *str != ' '  && *str != '\t' &&
-                   *str != '\n' && *str != '\r') {
-
-                modeline->data.clock += digits * (*str - '0');
-                digits /= 10;
-                str++;
-            }
-        }
-        str = parse_skip_whitespace(str);
-    }
+    str = parse_read_name(str, &(modeline->data.clock), 0);
+    if (!str) goto fail;
 
+    /* Read the mode timings */
     str = parse_read_integer(str, &(modeline->data.hdisplay));
     str = parse_read_integer(str, &(modeline->data.hsyncstart));
     str = parse_read_integer(str, &(modeline->data.hsyncend));
@@ -301,7 +297,7 @@ static nvModeLinePtr modeline_parse(const char *modeline_str)
 
 
     /* Parse modeline flags */
-    while ((str = parse_read_name(str, &tmp, ' ')) && strlen(tmp)) {
+    while ((str = parse_read_name(str, &tmp, 0)) && strlen(tmp)) {
         
         if (!xconfigNameCompare(tmp, "+hsync")) {
             modeline->data.flags |= XCONFIG_MODE_PHSYNC;
@@ -360,12 +356,59 @@ static nvModeLinePtr modeline_parse(const char *modeline_str)
         free(tmp);
     }
 
+
+    /*
+     * Calculate the vertical refresh rate of the modeline in Hz;
+     * divide by two for double scan modes (if the double scan
+     * modeline isn't broken; i.e., already has a correct vtotal), and
+     * multiply by two for interlaced modes (so that we report the
+     * field rate, rather than the frame rate)
+     */
+
+    htotal = (double) modeline->data.htotal;
+    vtotal = (double) modeline->data.vtotal;
+
+    pclk = strtod(modeline->data.clock, &nptr);
+
+    if ((pclk == 0.0) || !nptr || *nptr != '\0' || ((htotal * vtotal) == 0)) {
+        nv_warning_msg("Failed to compute the refresh rate "
+                       "for the modeline '%s'", str);
+        goto fail;
+    }
+
+    modeline->refresh_rate = (pclk * 1000000.0) / (htotal * vtotal);
+
+    factor = 1.0;
+
+    if ((modeline->data.flags & V_DBLSCAN) && !broken_doublescan_modelines) {
+        factor *= 0.5;
+    }
+
+    if (modeline->data.flags & V_INTERLACE) {
+        factor *= 2.0;
+    }
+
+    modeline->refresh_rate *= factor;
+
+    /* Restore the locale */
+    if (old_locale) {
+        setlocale(LC_NUMERIC, old_locale);
+        free(old_locale);
+    }
+
     return modeline;
 
 
     /* Handle failures */
  fail:
     free(modeline);
+
+    /* Restore the locale */
+    if (old_locale) {
+        setlocale(LC_NUMERIC, old_locale);
+        free(old_locale);
+    }
+
     return NULL;
 
 } /* modeline_parse() */
@@ -407,7 +450,7 @@ nvModePtr mode_parse(nvDisplayPtr display, const char *mode_str)
 
 
     /* Read the mode name */
-    str = parse_read_name(str, &mode_name, ' ');
+    str = parse_read_name(str, &mode_name, 0);
     if (!str || !mode_name) goto fail;
 
 
@@ -708,6 +751,93 @@ int display_find_closest_mode_matching_modeline(nvDisplayPtr display,
 
 
 
+/** modeline_free() *************************************
+ *
+ * Helper function that frees an nvModeLinePtr and
+ * associated memory.
+ *
+ **/
+void modeline_free(nvModeLinePtr m)
+{
+    if (m->xconfig_name) {
+        free(m->xconfig_name);
+    }
+
+    if (m->data.identifier) {
+        free(m->data.identifier);
+    }
+
+    if (m->data.comment) {
+        free(m->data.comment);
+    }
+
+    if (m->data.clock) {
+        free(m->data.clock);
+    }
+
+    free(m);
+}
+
+
+
+/** modelines_match() *************************************
+ *
+ * Helper function that returns True or False based on whether
+ * the modeline arguments match each other.
+ *
+ **/
+Bool modelines_match(nvModeLinePtr modeline1,
+                     nvModeLinePtr modeline2)
+{
+    if (!modeline1 || !modeline2) {
+        return FALSE;
+    }
+
+    if (!g_ascii_strcasecmp(modeline1->data.clock, modeline2->data.clock) &&
+        modeline1->data.hdisplay == modeline2->data.hdisplay &&
+        modeline1->data.hsyncstart == modeline2->data.hsyncstart &&
+        modeline1->data.hsyncend == modeline2->data.hsyncend &&
+        modeline1->data.htotal == modeline2->data.htotal &&
+        modeline1->data.vdisplay == modeline2->data.vdisplay &&
+        modeline1->data.vsyncstart == modeline2->data.vsyncstart &&
+        modeline1->data.vsyncend == modeline2->data.vsyncend &&
+        modeline1->data.vtotal == modeline2->data.vtotal &&
+        modeline1->data.vscan == modeline2->data.vscan &&
+        modeline1->data.flags == modeline2->data.flags &&
+        modeline1->data.hskew == modeline2->data.hskew &&
+        !g_ascii_strcasecmp(modeline1->data.identifier, 
+                            modeline2->data.identifier)) {
+            return TRUE;
+    } else {
+        return FALSE;
+    }
+} /* modelines_match() */
+
+
+
+/** display_has_modeline() *******************************************
+ *
+ * Helper function that returns TRUE or FALSE based on whether
+ * the display passed as argument supports the given modeline.
+ *
+ **/
+Bool display_has_modeline(nvDisplayPtr display,
+                          nvModeLinePtr modeline)
+{
+    nvModeLinePtr m;
+
+    for (m = display->modelines; m; m = m->next) {
+         if (modelines_match(m, modeline)) {
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+
+} /* display_has_modeline() */
+
+
+
 /** display_remove_modelines() ***************************************
  *
  * Clears the display device's modeline list.
@@ -721,7 +851,7 @@ static void display_remove_modelines(nvDisplayPtr display)
         while (display->modelines) {
             modeline = display->modelines;
             display->modelines = display->modelines->next;
-            free(modeline);
+            modeline_free(modeline);
         }
         display->num_modelines = 0;
     }
@@ -741,7 +871,29 @@ Bool display_add_modelines_from_server(nvDisplayPtr display, gchar **err_str)
     char *modeline_strs = NULL;
     char *str;
     int len;
-    ReturnStatus ret;
+    ReturnStatus ret, ret1;
+    int major = 0, minor = 0;
+    int broken_doublescan_modelines;
+
+    /*
+     * check the version of the NV-CONTROL protocol -- versions <=
+     * 1.13 had a bug in how they reported double scan modelines
+     * (vsyncstart, vsyncend, and vtotal were doubled); determine
+     * if this X server has this bug, so that we can use
+     * broken_doublescan_modelines to correctly compute the
+     * refresh rate.
+     */
+    broken_doublescan_modelines = 1;
+
+    ret = NvCtrlGetAttribute(display->gpu->handle,
+                             NV_CTRL_ATTR_NV_MAJOR_VERSION, &major);
+    ret1 = NvCtrlGetAttribute(display->gpu->handle,
+                              NV_CTRL_ATTR_NV_MINOR_VERSION, &minor);
+
+    if ((ret == NvCtrlSuccess) && (ret1 == NvCtrlSuccess) &&
+        ((major > 1) || ((major == 1) && (minor > 13)))) {
+        broken_doublescan_modelines = 0;
+    }
 
 
     /* Free any old mode lines */
@@ -769,7 +921,7 @@ Bool display_add_modelines_from_server(nvDisplayPtr display, gchar **err_str)
     str = modeline_strs;
     while (strlen(str)) {
 
-        modeline = modeline_parse(str);
+        modeline = modeline_parse(str, broken_doublescan_modelines);
         if (!modeline) {
             *err_str = g_strdup_printf("Failed to parse the following "
                                        "modeline of display device\n"
@@ -783,7 +935,7 @@ Bool display_add_modelines_from_server(nvDisplayPtr display, gchar **err_str)
             nv_error_msg(*err_str);
             goto fail;
         }
-        
+
         /* Add the modeline at the end of the display's modeline list */
         display->modelines = (nvModeLinePtr)xconfigAddListItem
             ((GenericListPtr)display->modelines, (GenericListPtr)modeline);
@@ -2072,6 +2224,12 @@ static Bool layout_add_gpu_from_server(nvLayoutPtr layout, unsigned int gpu_id,
         goto fail;
     }
 
+    ret = NvCtrlGetAttribute(gpu->handle, NV_CTRL_DEPTH_30_ALLOWED,
+                             &(gpu->allow_depth_30));
+
+    if (ret != NvCtrlSuccess) {
+        gpu->allow_depth_30 = FALSE;
+    }
 
     /* Add the display devices to the GPU */
     if (!gpu_add_displays_from_server(gpu, err_str)) {
